@@ -1,6 +1,9 @@
 // test.cpp - written and placed in the public domain by Wei Dai
 
+#define _CRT_SECURE_NO_DEPRECATE
 #define CRYPTOPP_DEFAULT_NO_DLL
+#define CRYPTOPP_ENABLE_NAMESPACE_WEAK 1
+
 #include "dll.h"
 #include "md5.h"
 #include "ripemd.h"
@@ -13,6 +16,8 @@
 #include "socketft.h"
 #include "wait.h"
 #include "factory.h"
+#include "whrlpool.h"
+#include "tiger.h"
 
 #include "validate.h"
 #include "bench.h"
@@ -37,11 +42,18 @@
 #include <console.h>
 #endif
 
+#ifdef __BORLANDC__
+#pragma comment(lib, "cryptlib_bds.lib")
+#pragma comment(lib, "ws2_32.lib")
+#endif
+
 USING_NAMESPACE(CryptoPP)
 USING_NAMESPACE(std)
 
 const int MAX_PHRASE_LENGTH=250;
 
+void RegisterFactories();
+
 void GenerateRSAKey(unsigned int keyLength, const char *privFilename, const char *pubFilename, const char *seed);
 string RSAEncryptString(const char *pubFilename, const char *seed, const char *message);
 string RSADecryptString(const char *privFilename, const char *ciphertext);
@@ -82,11 +94,7 @@ bool Validate(int, bool, const char *);
 
 int (*AdhocTest)(int argc, char *argv[]) = NULL;
 
-#ifdef __BCPLUSPLUS__
-int cmain(int argc, char *argv[])
-#else
-int CRYPTOPP_CDECL main(int argc, char *argv[])
-#endif
+int CRYPTOPP_API main(int argc, char *argv[])
 {
 #ifdef _CRTDBG_LEAK_CHECK_DF
 	// Turn on leak-checking
@@ -101,6 +109,8 @@ int CRYPTOPP_CDECL main(int argc, char *argv[])
 
 	try
 	{
+		RegisterFactories();
+
 		std::string command, executableName, macFilename;
 
 		if (argc < 2)
@@ -166,48 +176,73 @@ int CRYPTOPP_CDECL main(int argc, char *argv[])
 			fs.TransferAllTo(mt);
 			cout << "Maurer Test Value: " << mt.GetTestValue() << endl;
 		}
-#ifdef CRYPTOPP_WIN32_AVAILABLE
 		else if (command == "mac_dll")
 		{
-			HMODULE hModule = LoadLibrary(argv[2]);
-			PGetPowerUpSelfTestStatus pGetPowerUpSelfTestStatus = (PGetPowerUpSelfTestStatus)GetProcAddress(hModule, "?GetPowerUpSelfTestStatus@CryptoPP@@YG?AW4PowerUpSelfTestStatus@1@XZ");
-			PGetActualMacAndLocation pGetActualMacAndLocation = (PGetActualMacAndLocation)GetProcAddress(hModule, "?GetActualMacAndLocation@CryptoPP@@YGPBEAAI0@Z");
-
-			PowerUpSelfTestStatus status = pGetPowerUpSelfTestStatus();
-			if (status == POWER_UP_SELF_TEST_PASSED)
+			// sanity check on file size
+			std::fstream dllFile(argv[2], ios::in | ios::out | ios::binary);
+			std::ifstream::pos_type fileEnd = dllFile.seekg(0, std::ios_base::end).tellg();
+			if (fileEnd > 20*1000*1000)
 			{
-				cout << "Crypto++ DLL MAC is valid. Nothing to do.\n";
-				return 0;
-			}
-
-			unsigned int macSize, macFileLocation;
-			const byte *pMac = pGetActualMacAndLocation(macSize, macFileLocation);
-			
-			if (macFileLocation == 0)
-			{
-				cerr << "Could not find MAC location in Crypto++ DLL.\n";
+				cerr << "Input file too large (more than 20 MB).\n";
 				return 1;
 			}
-			else
+
+			// read file into memory
+			unsigned int fileSize = (unsigned int)fileEnd;
+			SecByteBlock buf(fileSize);
+			dllFile.seekg(0, std::ios_base::beg);
+			dllFile.read((char *)buf.begin(), fileSize);
+
+			// find positions of relevant sections in the file, based on version 8 of documentation from http://www.microsoft.com/whdc/system/platform/firmware/PECOFF.mspx
+			word32 coffPos = *(word16 *)(buf+0x3c);
+			word32 optionalHeaderPos = coffPos + 24;
+			word16 optionalHeaderMagic = *(word16 *)(buf+optionalHeaderPos);
+			if (optionalHeaderMagic != 0x10b && optionalHeaderMagic != 0x20b)
 			{
-				SecByteBlock mac(pMac, macSize);	// copy MAC before freeing the DLL
-				BOOL r = FreeLibrary(hModule);
-				cout << "Placing MAC in file " << argv[2] << ", location " << macFileLocation << ".\n";
-				std::ofstream dllFile(argv[2], ios::in | ios::out | ios::binary);
-				dllFile.seekp(macFileLocation);
-				dllFile.write((const char *)mac.data(), macSize);
-				if (!dllFile.good())
-				{
-					cerr << "Error writing file.\n";
-					return 1;
-				}
+				cerr << "Target file is not a PE32 or PE32+ image.\n";
+				return 3;
 			}
+			word32 checksumPos = optionalHeaderPos + 64;
+			word32 certificateTableDirectoryPos = optionalHeaderPos + (optionalHeaderMagic == 0x10b ? 128 : 144);
+			word32 certificateTablePos = *(word32 *)(buf+certificateTableDirectoryPos);
+			word32 certificateTableSize = *(word32 *)(buf+certificateTableDirectoryPos+4);
+			if (certificateTableSize != 0)
+				cerr << "Warning: certificate table (IMAGE_DIRECTORY_ENTRY_SECURITY) of target image is not empty.\n";
+
+			// find where to place computed MAC
+			byte mac[] = CRYPTOPP_DUMMY_DLL_MAC;
+			byte *found = std::search(buf.begin(), buf.end(), mac+0, mac+sizeof(mac));
+			if (found == buf.end())
+			{
+				cerr << "MAC placeholder not found. Possibly the actual MAC was already placed.\n";
+				return 2;
+			}
+			word32 macPos = (unsigned int)(found-buf.begin());
+
+			// compute MAC
+			member_ptr<MessageAuthenticationCode> pMac(NewIntegrityCheckingMAC());
+			assert(pMac->DigestSize() == sizeof(mac));
+			MeterFilter f(new HashFilter(*pMac, new ArraySink(mac, sizeof(mac))));
+			f.AddRangeToSkip(0, checksumPos, 4);
+			f.AddRangeToSkip(0, certificateTableDirectoryPos, 8);
+			f.AddRangeToSkip(0, macPos, sizeof(mac));
+			f.AddRangeToSkip(0, certificateTablePos, certificateTableSize);
+			f.PutMessageEnd(buf.begin(), buf.size());
+
+			// place MAC
+			cout << "Placing MAC in file " << argv[2] << ", location " << macPos << ".\n";
+			dllFile.seekg(macPos, std::ios_base::beg);
+			dllFile.write((char *)mac, sizeof(mac));
 		}
-#endif
 		else if (command == "m")
 			DigestFile(argv[2]);
 		else if (command == "tv")
-			return !RunTestDataFile(argv[2]);
+		{
+			std::string fname = argv[2];
+			if (fname.find(".txt") == std::string::npos)
+				fname = "TestVectors/" + fname + ".txt";
+			return !RunTestDataFile(fname.c_str());
+		}
 		else if (command == "t")
 		{
 			// VC60 workaround: use char array instead of std::string to workaround MSVC's getline bug
@@ -262,9 +297,9 @@ int CRYPTOPP_CDECL main(int argc, char *argv[])
 		else if (command == "v")
 			return !Validate(argc>2 ? atoi(argv[2]) : 0, argv[1][1] == 'v', argc>3 ? argv[3] : NULL);
 		else if (command == "b")
-			BenchmarkAll(argc<3 ? 1 : atof(argv[2]));
+			BenchmarkAll(argc<3 ? 1 : atof(argv[2]), argc<4 ? 0 : atof(argv[3])*1e9);
 		else if (command == "b2")
-			BenchmarkAll2(argc<3 ? 1 : atof(argv[2]));
+			BenchmarkAll2(argc<3 ? 1 : atof(argv[2]), argc<4 ? 0 : atof(argv[3])*1e9);
 		else if (command == "z")
 			GzipFile(argv[3], argv[4], argv[2][0]-'0');
 		else if (command == "u")
@@ -291,9 +326,13 @@ int CRYPTOPP_CDECL main(int argc, char *argv[])
 			AES_CTR_Encrypt(argv[2], argv[3], argv[4], argv[5]);
 		else if (command == "h")
 		{
-			FileSource usage(PKGDATADIR "usage.dat", true, new FileSink(cout));
+			FileSource usage("usage.dat", true, new FileSink(cout));
 			return 1;
 		}
+		else if (command == "V")
+		{
+			cout << "5.5" << endl;
+		}
 		else
 		{
 			cerr << "Unrecognized command. Run \"cryptest h\" to obtain usage information.\n";
@@ -316,7 +355,7 @@ int CRYPTOPP_CDECL main(int argc, char *argv[])
 void FIPS140_GenerateRandomFiles()
 {
 #ifdef OS_RNG_AVAILABLE
-	AutoSeededX917RNG<DES_EDE3> rng;
+	DefaultAutoSeededRNG rng;
 	RandomNumberStore store(rng, ULONG_MAX);
 
 	for (unsigned int i=0; i<100000; i++)
@@ -330,12 +369,12 @@ void FIPS140_GenerateRandomFiles()
 SecByteBlock HexDecodeString(const char *hex)
 {
 	StringSource ss(hex, true, new HexDecoder);
-	SecByteBlock result(ss.MaxRetrievable());
+	SecByteBlock result((size_t)ss.MaxRetrievable());
 	ss.Get(result, result.size());
 	return result;
 }
 
-RandomPool & GlobalRNG()
+RandomNumberGenerator & GlobalRNG()
 {
 	static RandomPool randomPool;
 	return randomPool;
@@ -344,7 +383,7 @@ RandomPool & GlobalRNG()
 void GenerateRSAKey(unsigned int keyLength, const char *privFilename, const char *pubFilename, const char *seed)
 {
 	RandomPool randPool;
-	randPool.Put((byte *)seed, strlen(seed));
+	randPool.IncorporateEntropy((byte *)seed, strlen(seed));
 
 	RSAES_OAEP_SHA_Decryptor priv(randPool, keyLength);
 	HexEncoder privFile(new FileSink(privFilename));
@@ -363,7 +402,7 @@ string RSAEncryptString(const char *pubFilename, const char *seed, const char *m
 	RSAES_OAEP_SHA_Encryptor pub(pubFile);
 
 	RandomPool randPool;
-	randPool.Put((byte *)seed, strlen(seed));
+	randPool.IncorporateEntropy((byte *)seed, strlen(seed));
 
 	string result;
 	StringSource(message, true, new PK_EncryptorFilter(randPool, pub, new HexEncoder(new StringSink(result))));
@@ -383,14 +422,14 @@ string RSADecryptString(const char *privFilename, const char *ciphertext)
 void RSASignFile(const char *privFilename, const char *messageFilename, const char *signatureFilename)
 {
 	FileSource privFile(privFilename, true, new HexDecoder);
-	RSASSA_PKCS1v15_SHA_Signer priv(privFile);
+	RSASS<PKCS1v15, SHA>::Signer priv(privFile);
 	FileSource f(messageFilename, true, new SignerFilter(GlobalRNG(), priv, new HexEncoder(new FileSink(signatureFilename))));
 }
 
 bool RSAVerifyFile(const char *pubFilename, const char *messageFilename, const char *signatureFilename)
 {
 	FileSource pubFile(pubFilename, true, new HexDecoder);
-	RSASSA_PKCS1v15_SHA_Verifier pub(pubFile);
+	RSASS<PKCS1v15, SHA>::Verifier pub(pubFile);
 
 	FileSource signatureFile(signatureFilename, true, new HexDecoder);
 	if (signatureFile.MaxRetrievable() != pub.SignatureLength())
@@ -407,28 +446,40 @@ bool RSAVerifyFile(const char *pubFilename, const char *messageFilename, const c
 
 void DigestFile(const char *filename)
 {
-	MD5 md5;
-	SHA sha;
+	SHA1 sha;
 	RIPEMD160 ripemd;
 	SHA256 sha256;
-	HashFilter md5Filter(md5), shaFilter(sha), ripemdFilter(ripemd), sha256Filter(sha256);
+#ifdef WORD64_AVAILABLE
+	Tiger tiger;
+	SHA512 sha512;
+	Whirlpool whirlpool;
+	vector_member_ptrs<HashFilter> filters(6);
+	filters[0].reset(new HashFilter(sha));
+	filters[1].reset(new HashFilter(ripemd));
+	filters[2].reset(new HashFilter(tiger));
+	filters[3].reset(new HashFilter(sha256));
+	filters[4].reset(new HashFilter(sha512));
+	filters[5].reset(new HashFilter(whirlpool));
+#else
+	vector_member_ptrs<HashFilter> filters(3);
+	filters[0].reset(new HashFilter(sha));
+	filters[1].reset(new HashFilter(ripemd));
+	filters[2].reset(new HashFilter(sha256));
+#endif
 
 	auto_ptr<ChannelSwitch> channelSwitch(new ChannelSwitch);
-	channelSwitch->AddDefaultRoute(md5Filter);
-	channelSwitch->AddDefaultRoute(shaFilter);
-	channelSwitch->AddDefaultRoute(ripemdFilter);
-	channelSwitch->AddDefaultRoute(sha256Filter);
+	size_t i;
+	for (i=0; i<filters.size(); i++)
+		channelSwitch->AddDefaultRoute(*filters[i]);
 	FileSource(filename, true, channelSwitch.release());
 
 	HexEncoder encoder(new FileSink(cout), false);
-	cout << "\nMD5: ";
-	md5Filter.TransferTo(encoder);
-	cout << "\nSHA-1: ";
-	shaFilter.TransferTo(encoder);
-	cout << "\nRIPEMD-160: ";
-	ripemdFilter.TransferTo(encoder);
-	cout << "\nSHA-256: ";
-	sha256Filter.TransferTo(encoder);
+	for (i=0; i<filters.size(); i++)
+	{
+		cout << filters[i]->AlgorithmName() << ": ";
+		filters[i]->TransferTo(encoder);
+		cout << "\n";
+	}
 }
 
 void HmacFile(const char *hexKey, const char *file)
@@ -493,7 +544,7 @@ void SecretShareFile(int threshold, int nShares, const char *filename, const cha
 	assert(nShares<=1000);
 
 	RandomPool rng;
-	rng.Put((byte *)seed, strlen(seed));
+	rng.IncorporateEntropy((byte *)seed, strlen(seed));
 
 	ChannelSwitch *channelSwitch;
 	FileSource source(filename, false, new SecretSharing(rng, threshold, nShares, channelSwitch = new ChannelSwitch));
@@ -684,8 +735,8 @@ void ForwardTcpPort(const char *sourcePortName, const char *destinationHost, con
 	{
 		waitObjects.Clear();
 
-		out.GetWaitObjects(waitObjects);
-		in.GetWaitObjects(waitObjects);
+		out.GetWaitObjects(waitObjects, CallStack("ForwardTcpPort - out", NULL));
+		in.GetWaitObjects(waitObjects, CallStack("ForwardTcpPort - in", NULL));
 
 		waitObjects.Wait(INFINITE_TIME);
 
@@ -723,10 +774,11 @@ bool Validate(int alg, bool thorough, const char *seed)
 	}
 
 	cout << "Using seed: " << seed << endl << endl;
-	GlobalRNG().Put((const byte *)seed, strlen(seed));
+	GlobalRNG().IncorporateEntropy((const byte *)seed, strlen(seed));
 
 	switch (alg)
 	{
+	case 0: result = ValidateAll(thorough); break;
 	case 1: result = TestSettings(); break;
 	case 2: result = TestOS_RNG(); break;
 	case 3: result = ValidateMD5(); break;
@@ -743,19 +795,19 @@ bool Validate(int alg, bool thorough, const char *seed)
 	case 14: result = ValidateRSA(); break;
 	case 15: result = ValidateElGamal(); break;
 	case 16: result = ValidateDSA(thorough); break;
-	case 17: result = ValidateHAVAL(); break;
+//	case 17: result = ValidateHAVAL(); break;
 	case 18: result = ValidateSAFER(); break;
 	case 19: result = ValidateLUC(); break;
 	case 20: result = ValidateRabin(); break;
 //	case 21: result = ValidateBlumGoldwasser(); break;
 	case 22: result = ValidateECP(); break;
 	case 23: result = ValidateEC2N(); break;
-	case 24: result = ValidateMD5MAC(); break;
+//	case 24: result = ValidateMD5MAC(); break;
 	case 25: result = ValidateGOST(); break;
 	case 26: result = ValidateTiger(); break;
 	case 27: result = ValidateRIPEMD(); break;
 	case 28: result = ValidateHMAC(); break;
-	case 29: result = ValidateXMACC(); break;
+//	case 29: result = ValidateXMACC(); break;
 	case 30: result = ValidateSHARK(); break;
 	case 32: result = ValidateLUC_DH(); break;
 	case 33: result = ValidateLUC_DL(); break;
@@ -789,7 +841,10 @@ bool Validate(int alg, bool thorough, const char *seed)
 	case 61: result = ValidateCamellia(); break;
 	case 62: result = ValidateWhirlpool(); break;
 	case 63: result = ValidateTTMAC(); break;
-	default: result = ValidateAll(thorough); break;
+	case 64: result = ValidateSalsa(); break;
+	case 65: result = ValidateSosemanuk(); break;
+	case 66: result = ValidateVMAC(); break;
+	default: return false;
 	}
 
 	time_t endTime = time(NULL);
