@@ -2,6 +2,7 @@
 
 #include "pch.h"
 
+#define CRYPTOPP_ENABLE_NAMESPACE_WEAK 1
 #include "files.h"
 #include "hex.h"
 #include "base32.h"
@@ -33,6 +34,7 @@
 #include "camellia.h"
 #include "osrng.h"
 #include "zdeflate.h"
+#include "cpu.h"
 
 #include <stdlib.h>
 #include <time.h>
@@ -56,15 +58,12 @@ bool ValidateAll(bool thorough)
 	pass=ValidateMD5() && pass;
 	pass=ValidateSHA() && pass;
 	pass=ValidateSHA2() && pass;
-	pass=ValidateHAVAL() && pass;
 	pass=ValidateTiger() && pass;
 	pass=ValidateRIPEMD() && pass;
 	pass=ValidatePanama() && pass;
 	pass=ValidateWhirlpool() && pass;
 
-	pass=ValidateMD5MAC() && pass;
 	pass=ValidateHMAC() && pass;
-	pass=ValidateXMACC() && pass;
 	pass=ValidateTTMAC() && pass;
 
 	pass=ValidatePBKDF() && pass;
@@ -91,6 +90,9 @@ bool ValidateAll(bool thorough)
 	pass=ValidateSerpent() && pass;
 	pass=ValidateSHACAL2() && pass;
 	pass=ValidateCamellia() && pass;
+	pass=ValidateSalsa() && pass;
+	pass=ValidateSosemanuk() && pass;
+	pass=ValidateVMAC() && pass;
 
 	pass=ValidateBBS() && pass;
 	pass=ValidateDH() && pass;
@@ -188,7 +190,7 @@ bool TestSettings()
 		pass = false;
 	}
 	cout << "sizeof(word64) == " << sizeof(word64) << endl;
-#elif CRYPTOPP_NATIVE_DWORD_AVAILABLE
+#elif defined(CRYPTOPP_NATIVE_DWORD_AVAILABLE)
 	if (sizeof(dword) >= 8)
 	{
 		cout << "FAILED:  sizeof(dword) >= 8, but WORD64_AVAILABLE not defined" << endl;
@@ -198,17 +200,50 @@ bool TestSettings()
 		cout << "passed:  word64 not available" << endl;
 #endif
 
-#ifdef CRYPTOPP_NATIVE_DWORD_AVAILABLE
-	if (sizeof(dword) == 2*sizeof(word))
+#ifdef CRYPTOPP_WORD128_AVAILABLE
+	if (sizeof(word128) == 16)
 		cout << "passed:  ";
 	else
 	{
 		cout << "FAILED:  ";
 		pass = false;
 	}
-	cout << "sizeof(word) == " << sizeof(word) << ", sizeof(dword) == " << sizeof(dword) << endl;
+	cout << "sizeof(word128) == " << sizeof(word128) << endl;
 #endif
 
+	if (sizeof(word) == 2*sizeof(hword)
+#ifdef CRYPTOPP_NATIVE_DWORD_AVAILABLE
+		&& sizeof(dword) == 2*sizeof(word)
+#endif
+		)
+		cout << "passed:  ";
+	else
+	{
+		cout << "FAILED:  ";
+		pass = false;
+	}
+	cout << "sizeof(hword) == " << sizeof(hword) << ", sizeof(word) == " << sizeof(word);
+#ifdef CRYPTOPP_NATIVE_DWORD_AVAILABLE
+	cout << ", sizeof(dword) == " << sizeof(dword);
+#endif
+	cout << endl;
+
+	bool hasMMX = HasMMX();
+	bool hasSSE2 = HasSSE2();
+	bool hasSSSE3 = HasSSSE3();
+	bool isP4 = IsP4();
+	int cacheLineSize = GetCacheLineSize();
+
+	if ((isP4 && (!hasMMX || !hasSSE2)) || (hasSSE2 && !hasMMX) || (cacheLineSize < 16 || cacheLineSize > 256 || !IsPowerOf2(cacheLineSize)))
+	{
+		cout << "FAILED:  ";
+		pass = false;
+	}
+	else
+		cout << "passed:  ";
+
+	cout << "hasMMX == " << hasMMX << ", hasSSE2 == " << hasSSE2 << ", hasSSSE3 == " << hasSSSE3 << ", isP4 == " << isP4 << ", cacheLineSize == " << cacheLineSize;
+
 	if (!pass)
 	{
 		cout << "Some critical setting in config.h is in error.  Please fix it and recompile." << endl;
@@ -251,8 +286,9 @@ bool TestOS_RNG()
 		}
 		else
 			cout << "passed:";
-		cout << "  it took " << t1 << " seconds to generate " << total << " bytes" << endl;
+		cout << "  it took " << long(t1) << " seconds to generate " << total << " bytes" << endl;
 
+#if 0	// disable this part. it's causing an unpredictable pause during the validation testing
 		if (t1 < 2)
 		{
 			// that was fast, are we really blocking?
@@ -273,17 +309,16 @@ bool TestOS_RNG()
 				total += 1;
 				length += 1;
 			}
-			// turn off this test because it fails on several systems, including Darwin
-			// they don't block, or gather entropy too fast?
-			if (false) // (length > 1024)
+			if (length > 1024)
 			{
 				cout << "FAILED:";
 				pass = false;
 			}
 			else
 				cout << "passed:";
-			cout << "  it generated " << length << " bytes in " << time(NULL) - t << " seconds" << endl;
+			cout << "  it generated " << length << " bytes in " << long(time(NULL) - t) << " seconds" << endl;
 		}
+#endif
 
 		test.AttachedTransformation()->MessageEnd();
 
@@ -294,7 +329,7 @@ bool TestOS_RNG()
 		}
 		else
 			cout << "passed:";
-		cout << "  " << total << " generated bytes compressed to " << sink->TotalPutLength() << " bytes by DEFLATE" << endl;
+		cout << "  " << total << " generated bytes compressed to " << (size_t)sink->TotalPutLength() << " bytes by DEFLATE" << endl;
 	}
 	else
 		cout << "\nNo operating system provided blocking random number generator, skipping test." << endl;
@@ -319,7 +354,7 @@ bool TestOS_RNG()
 		}
 		else
 			cout << "passed:";
-		cout << "  100000 generated bytes compressed to " << sink->TotalPutLength() << " bytes by DEFLATE" << endl;
+		cout << "  100000 generated bytes compressed to " << (size_t)sink->TotalPutLength() << " bytes by DEFLATE" << endl;
 	}
 	else
 		cout << "\nNo operating system provided nonblocking random number generator, skipping test." << endl;
@@ -408,18 +443,19 @@ bool BlockTransformationTest(const CipherFactory &cg, BufferedTransformation &va
 class FilterTester : public Unflushable<Sink>
 {
 public:
-	FilterTester(const byte *validOutput, unsigned int outputLen)
+	FilterTester(const byte *validOutput, size_t outputLen)
 		: validOutput(validOutput), outputLen(outputLen), counter(0), fail(false) {}
 	void PutByte(byte inByte)
 	{
 		if (counter >= outputLen || validOutput[counter] != inByte)
 		{
+			std::cerr << "incorrect output " << counter << ", " << (word16)validOutput[counter] << ", " << (word16)inByte << "\n";
 			fail = true;
 			assert(false);
 		}
 		counter++;
 	}
-	unsigned int Put2(const byte *inString, unsigned int length, int messageEnd, bool blocking)
+	size_t Put2(const byte *inString, size_t length, int messageEnd, bool blocking)
 	{
 		while (length--)
 			FilterTester::PutByte(*inString++);
@@ -439,18 +475,18 @@ public:
 	}
 
 	const byte *validOutput;
-	unsigned int outputLen, counter;
+	size_t outputLen, counter;
 	bool fail;
 };
 
-bool TestFilter(BufferedTransformation &bt, const byte *in, unsigned int inLen, const byte *out, unsigned int outLen)
+bool TestFilter(BufferedTransformation &bt, const byte *in, size_t inLen, const byte *out, size_t outLen)
 {
 	FilterTester *ft;
 	bt.Attach(ft = new FilterTester(out, outLen));
 
 	while (inLen)
 	{
-		unsigned int randomLen = GlobalRNG().GenerateWord32(0, inLen);
+		size_t randomLen = GlobalRNG().GenerateWord32(0, (word32)inLen);
 		bt.Put(in, randomLen);
 		in += randomLen;
 		inLen -= randomLen;
@@ -463,12 +499,12 @@ bool ValidateDES()
 {
 	cout << "\nDES validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("descert.dat"), true, new HexDecoder);
+	FileSource valdata("descert.dat", true, new HexDecoder);
 	bool pass = BlockTransformationTest(FixedRoundsCipherFactory<DESEncryption, DESDecryption>(), valdata);
 
 	cout << "\nTesting EDE2, EDE3, and XEX3 variants...\n\n";
 
-	FileSource valdata1(PKGDATA("3desval.dat"), true, new HexDecoder);
+	FileSource valdata1("3desval.dat", true, new HexDecoder);
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<DES_EDE2_Encryption, DES_EDE2_Decryption>(), valdata1, 1) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<DES_EDE3_Encryption, DES_EDE3_Decryption>(), valdata1, 1) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<DES_XEX3_Encryption, DES_XEX3_Decryption>(), valdata1, 1) && pass;
@@ -478,15 +514,13 @@ bool ValidateDES()
 
 bool TestModeIV(SymmetricCipher &e, SymmetricCipher &d)
 {
-	SecByteBlock lastIV;
+	SecByteBlock lastIV, iv(e.IVSize());
 	StreamTransformationFilter filter(e, new StreamTransformationFilter(d));
 	byte plaintext[20480];
 
 	for (unsigned int i=1; i<sizeof(plaintext); i*=2)
 	{
-		SecByteBlock iv(e.IVSize());
-		e.GetNextIV(iv);
-
+		e.GetNextIV(GlobalRNG(), iv);
 		if (iv == lastIV)
 			return false;
 		else
@@ -784,7 +818,7 @@ bool ValidateIDEA()
 {
 	cout << "\nIDEA validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("ideaval.dat"), true, new HexDecoder);
+	FileSource valdata("ideaval.dat", true, new HexDecoder);
 	return BlockTransformationTest(FixedRoundsCipherFactory<IDEAEncryption, IDEADecryption>(), valdata);
 }
 
@@ -792,7 +826,7 @@ bool ValidateSAFER()
 {
 	cout << "\nSAFER validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("saferval.dat"), true, new HexDecoder);
+	FileSource valdata("saferval.dat", true, new HexDecoder);
 	bool pass = true;
 	pass = BlockTransformationTest(VariableRoundsCipherFactory<SAFER_K_Encryption, SAFER_K_Decryption>(8,6), valdata, 4) && pass;
 	pass = BlockTransformationTest(VariableRoundsCipherFactory<SAFER_K_Encryption, SAFER_K_Decryption>(16,12), valdata, 4) && pass;
@@ -805,7 +839,7 @@ bool ValidateRC2()
 {
 	cout << "\nRC2 validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("rc2val.dat"), true, new HexDecoder);
+	FileSource valdata("rc2val.dat", true, new HexDecoder);
 	HexEncoder output(new FileSink(cout));
 	SecByteBlock plain(RC2Encryption::BLOCKSIZE), cipher(RC2Encryption::BLOCKSIZE), out(RC2Encryption::BLOCKSIZE), outplain(RC2Encryption::BLOCKSIZE);
 	SecByteBlock key(128);
@@ -969,39 +1003,39 @@ bool ValidateARC4()
 	0xc0};
 
 	// VC60 workaround: auto_ptr lacks reset()
-	member_ptr<ARC4> arc4;
+	member_ptr<Weak::ARC4> arc4;
 	bool pass=true, fail;
 	int i;
 
 	cout << "\nARC4 validation suite running...\n\n";
 
-	arc4.reset(new ARC4(Key0, sizeof(Key0)));
+	arc4.reset(new Weak::ARC4(Key0, sizeof(Key0)));
 	arc4->ProcessString(Input0, sizeof(Input0));
 	fail = memcmp(Input0, Output0, sizeof(Input0)) != 0;
 	cout << (fail ? "FAILED" : "passed") << "    Test 0" << endl;
 	pass = pass && !fail;
 
-	arc4.reset(new ARC4(Key1, sizeof(Key1)));
+	arc4.reset(new Weak::ARC4(Key1, sizeof(Key1)));
 	arc4->ProcessString(Key1, Input1, sizeof(Key1));
 	fail = memcmp(Output1, Key1, sizeof(Key1)) != 0;
 	cout << (fail ? "FAILED" : "passed") << "    Test 1" << endl;
 	pass = pass && !fail;
 
-	arc4.reset(new ARC4(Key2, sizeof(Key2)));
+	arc4.reset(new Weak::ARC4(Key2, sizeof(Key2)));
 	for (i=0, fail=false; i<sizeof(Input2); i++)
 		if (arc4->ProcessByte(Input2[i]) != Output2[i])
 			fail = true;
 	cout << (fail ? "FAILED" : "passed") << "    Test 2" << endl;
 	pass = pass && !fail;
 
-	arc4.reset(new ARC4(Key3, sizeof(Key3)));
+	arc4.reset(new Weak::ARC4(Key3, sizeof(Key3)));
 	for (i=0, fail=false; i<sizeof(Input3); i++)
 		if (arc4->ProcessByte(Input3[i]) != Output3[i])
 			fail = true;
 	cout << (fail ? "FAILED" : "passed") << "    Test 3" << endl;
 	pass = pass && !fail;
 
-	arc4.reset(new ARC4(Key4, sizeof(Key4)));
+	arc4.reset(new Weak::ARC4(Key4, sizeof(Key4)));
 	for (i=0, fail=false; i<sizeof(Input4); i++)
 		if (arc4->ProcessByte(Input4[i]) != Output4[i])
 			fail = true;
@@ -1015,7 +1049,7 @@ bool ValidateRC5()
 {
 	cout << "\nRC5 validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("rc5val.dat"), true, new HexDecoder);
+	FileSource valdata("rc5val.dat", true, new HexDecoder);
 	return BlockTransformationTest(VariableRoundsCipherFactory<RC5Encryption, RC5Decryption>(16, 12), valdata);
 }
 
@@ -1023,7 +1057,7 @@ bool ValidateRC6()
 {
 	cout << "\nRC6 validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("rc6val.dat"), true, new HexDecoder);
+	FileSource valdata("rc6val.dat", true, new HexDecoder);
 	bool pass = true;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<RC6Encryption, RC6Decryption>(16), valdata, 2) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<RC6Encryption, RC6Decryption>(24), valdata, 2) && pass;
@@ -1035,7 +1069,7 @@ bool ValidateMARS()
 {
 	cout << "\nMARS validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("marsval.dat"), true, new HexDecoder);
+	FileSource valdata("marsval.dat", true, new HexDecoder);
 	bool pass = true;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<MARSEncryption, MARSDecryption>(16), valdata, 4) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<MARSEncryption, MARSDecryption>(24), valdata, 3) && pass;
@@ -1047,7 +1081,7 @@ bool ValidateRijndael()
 {
 	cout << "\nRijndael validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("rijndael.dat"), true, new HexDecoder);
+	FileSource valdata("rijndael.dat", true, new HexDecoder);
 	bool pass = true;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<RijndaelEncryption, RijndaelDecryption>(16), valdata, 4) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<RijndaelEncryption, RijndaelDecryption>(24), valdata, 3) && pass;
@@ -1059,7 +1093,7 @@ bool ValidateTwofish()
 {
 	cout << "\nTwofish validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("twofishv.dat"), true, new HexDecoder);
+	FileSource valdata("twofishv.dat", true, new HexDecoder);
 	bool pass = true;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<TwofishEncryption, TwofishDecryption>(16), valdata, 4) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<TwofishEncryption, TwofishDecryption>(24), valdata, 3) && pass;
@@ -1071,7 +1105,7 @@ bool ValidateSerpent()
 {
 	cout << "\nSerpent validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("serpentv.dat"), true, new HexDecoder);
+	FileSource valdata("serpentv.dat", true, new HexDecoder);
 	bool pass = true;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<SerpentEncryption, SerpentDecryption>(16), valdata, 4) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<SerpentEncryption, SerpentDecryption>(24), valdata, 3) && pass;
@@ -1117,7 +1151,7 @@ bool ValidateThreeWay()
 {
 	cout << "\n3-WAY validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("3wayval.dat"), true, new HexDecoder);
+	FileSource valdata("3wayval.dat", true, new HexDecoder);
 	return BlockTransformationTest(FixedRoundsCipherFactory<ThreeWayEncryption, ThreeWayDecryption>(), valdata);
 }
 
@@ -1125,7 +1159,7 @@ bool ValidateGOST()
 {
 	cout << "\nGOST validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("gostval.dat"), true, new HexDecoder);
+	FileSource valdata("gostval.dat", true, new HexDecoder);
 	return BlockTransformationTest(FixedRoundsCipherFactory<GOSTEncryption, GOSTDecryption>(), valdata);
 }
 
@@ -1134,7 +1168,7 @@ bool ValidateSHARK()
 	cout << "\nSHARK validation suite running...\n\n";
 
 #ifdef WORD64_AVAILABLE
-	FileSource valdata(PKGDATA("sharkval.dat"), true, new HexDecoder);
+	FileSource valdata("sharkval.dat", true, new HexDecoder);
 	return BlockTransformationTest(FixedRoundsCipherFactory<SHARKEncryption, SHARKDecryption>(), valdata);
 #else
 	cout << "word64 not available, skipping SHARK validation." << endl;
@@ -1148,14 +1182,14 @@ bool ValidateCAST()
 
 	cout << "\nCAST-128 validation suite running...\n\n";
 
-	FileSource val128(PKGDATA("cast128v.dat"), true, new HexDecoder);
+	FileSource val128("cast128v.dat", true, new HexDecoder);
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST128Encryption, CAST128Decryption>(16), val128, 1) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST128Encryption, CAST128Decryption>(10), val128, 1) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST128Encryption, CAST128Decryption>(5), val128, 1) && pass;
 
 	cout << "\nCAST-256 validation suite running...\n\n";
 
-	FileSource val256(PKGDATA("cast256v.dat"), true, new HexDecoder);
+	FileSource val256("cast256v.dat", true, new HexDecoder);
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST256Encryption, CAST256Decryption>(16), val256, 1) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST256Encryption, CAST256Decryption>(24), val256, 1) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<CAST256Encryption, CAST256Decryption>(32), val256, 1) && pass;
@@ -1167,7 +1201,7 @@ bool ValidateSquare()
 {
 	cout << "\nSquare validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("squareva.dat"), true, new HexDecoder);
+	FileSource valdata("squareva.dat", true, new HexDecoder);
 	return BlockTransformationTest(FixedRoundsCipherFactory<SquareEncryption, SquareDecryption>(), valdata);
 }
 
@@ -1175,7 +1209,7 @@ bool ValidateSKIPJACK()
 {
 	cout << "\nSKIPJACK validation suite running...\n\n";
 
-	FileSource valdata(PKGDATA("skipjack.dat"), true, new HexDecoder);
+	FileSource valdata("skipjack.dat", true, new HexDecoder);
 	return BlockTransformationTest(FixedRoundsCipherFactory<SKIPJACKEncryption, SKIPJACKDecryption>(), valdata);
 }
 
@@ -1279,7 +1313,7 @@ bool ValidateSHACAL2()
 	cout << "\nSHACAL-2 validation suite running...\n\n";
 
 	bool pass = true;
-	FileSource valdata(PKGDATA("shacal2v.dat"), true, new HexDecoder);
+	FileSource valdata("shacal2v.dat", true, new HexDecoder);
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<SHACAL2Encryption, SHACAL2Decryption>(16), valdata, 4) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<SHACAL2Encryption, SHACAL2Decryption>(64), valdata, 10) && pass;
 	return pass;
@@ -1289,15 +1323,29 @@ bool ValidateCamellia()
 {
 	cout << "\nCamellia validation suite running...\n\n";
 
-#ifdef WORD64_AVAILABLE
 	bool pass = true;
-	FileSource valdata(PKGDATA("camellia.dat"), true, new HexDecoder);
+	FileSource valdata("camellia.dat", true, new HexDecoder);
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<CamelliaEncryption, CamelliaDecryption>(16), valdata, 15) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<CamelliaEncryption, CamelliaDecryption>(24), valdata, 15) && pass;
 	pass = BlockTransformationTest(FixedRoundsCipherFactory<CamelliaEncryption, CamelliaDecryption>(32), valdata, 15) && pass;
 	return pass;
-#else
-	cout << "word64 not available, skipping Camellia validation." << endl;
-	return true;
-#endif
+}
+
+bool ValidateSalsa()
+{
+	cout << "\nSalsa validation suite running...\n";
+
+	return RunTestDataFile("TestVectors/salsa.txt");
+}
+
+bool ValidateSosemanuk()
+{
+	cout << "\nSosemanuk validation suite running...\n";
+	return RunTestDataFile("TestVectors/sosemanuk.txt");
+}
+
+bool ValidateVMAC()
+{
+	cout << "\nVMAC validation suite running...\n";
+	return RunTestDataFile("TestVectors/vmac.txt");
 }
