@@ -2,6 +2,7 @@
 
 #include "pch.h"
 
+#define CRYPTOPP_ENABLE_NAMESPACE_WEAK 1
 #include "blumshub.h"
 #include "rsa.h"
 #include "md2.h"
@@ -38,11 +39,9 @@ class FixedRNG : public RandomNumberGenerator
 public:
 	FixedRNG(BufferedTransformation &source) : m_source(source) {}
 
-	byte GenerateByte()
+	void GenerateBlock(byte *output, size_t size)
 	{
-		byte b;
-		m_source.Get(b);
-		return b;
+		m_source.Get(output, size);
 	}
 
 private:
@@ -115,7 +114,7 @@ bool SignatureValidate(PK_Signer &priv, PK_Verifier &pub, bool thorough = false)
 	const int messageLen = 12;
 
 	SecByteBlock signature(priv.MaxSignatureLength());
-	unsigned int signatureLength = priv.SignMessage(GlobalRNG(), message, messageLen, signature);
+	size_t signatureLength = priv.SignMessage(GlobalRNG(), message, messageLen, signature);
 	fail = !pub.VerifyMessage(message, messageLen, signature, signatureLength);
 	pass = pass && !fail;
 
@@ -269,11 +268,11 @@ bool ValidateRSA()
 			"\x69\x94\xac\x04\xf3\x41\xb5\x7d\x05\x20\x2d\x42\x8f\xb2\xa2\x7b"
 			"\x5c\x77\xdf\xd9\xb1\x5b\xfc\x3d\x55\x93\x53\x50\x34\x10\xc1\xe1";
 
-		FileSource keys(PKGDATA("rsa512a.dat"), true, new HexDecoder);
-		RSASSA_PKCS1v15_MD2_Signer rsaPriv(keys);
-		RSASSA_PKCS1v15_MD2_Verifier rsaPub(rsaPriv);
+		FileSource keys("rsa512a.dat", true, new HexDecoder);
+		Weak::RSASSA_PKCS1v15_MD2_Signer rsaPriv(keys);
+		Weak::RSASSA_PKCS1v15_MD2_Verifier rsaPub(rsaPriv);
 
-		unsigned int signatureLength = rsaPriv.SignMessage(GlobalRNG(), (byte *)plain, strlen(plain), out);
+		size_t signatureLength = rsaPriv.SignMessage(GlobalRNG(), (byte *)plain, strlen(plain), out);
 		fail = memcmp(signature, out, 64) != 0;
 		pass = pass && !fail;
 
@@ -294,7 +293,7 @@ bool ValidateRSA()
 		cout << "invalid signature verification\n";
 	}
 	{
-		FileSource keys(PKGDATA("rsa1024.dat"), true, new HexDecoder);
+		FileSource keys("rsa1024.dat", true, new HexDecoder);
 		RSAES_PKCS1v15_Decryptor rsaPriv(keys);
 		RSAES_PKCS1v15_Encryptor rsaPub(rsaPriv);
 
@@ -315,10 +314,10 @@ bool ValidateRSA()
 		bq.Put(oaepSeed, 20);
 		FixedRNG rng(bq);
 
-		FileSource privFile(PKGDATA("rsa400pv.dat"), true, new HexDecoder);
-		FileSource pubFile(PKGDATA("rsa400pb.dat"), true, new HexDecoder);
+		FileSource privFile("rsa400pv.dat", true, new HexDecoder);
+		FileSource pubFile("rsa400pb.dat", true, new HexDecoder);
 		RSAES_OAEP_SHA_Decryptor rsaPriv;
-		rsaPriv.AccessKey().BERDecodeKey(privFile);
+		rsaPriv.AccessKey().BERDecodePrivateKey(privFile, false, 0);
 		RSAES_OAEP_SHA_Encryptor rsaPub(pubFile);
 
 		memset(out, 0, 50);
@@ -339,7 +338,7 @@ bool ValidateDH()
 {
 	cout << "\nDH validation suite running...\n\n";
 
-	FileSource f(PKGDATA("dh1024.dat"), true, new HexDecoder());
+	FileSource f("dh1024.dat", true, new HexDecoder());
 	DH dh(f);
 	return SimpleKeyAgreementValidate(dh);
 }
@@ -348,7 +347,7 @@ bool ValidateMQV()
 {
 	cout << "\nMQV validation suite running...\n\n";
 
-	FileSource f(PKGDATA("mqv1024.dat"), true, new HexDecoder());
+	FileSource f("mqv1024.dat", true, new HexDecoder());
 	MQV mqv(f);
 	return AuthenticatedKeyAgreementValidate(mqv);
 }
@@ -357,7 +356,7 @@ bool ValidateLUC_DH()
 {
 	cout << "\nLUC-DH validation suite running...\n\n";
 
-	FileSource f(PKGDATA("lucd512.dat"), true, new HexDecoder());
+	FileSource f("lucd512.dat", true, new HexDecoder());
 	LUC_DH dh(f);
 	return SimpleKeyAgreementValidate(dh);
 }
@@ -366,7 +365,7 @@ bool ValidateXTR_DH()
 {
 	cout << "\nXTR-DH validation suite running...\n\n";
 
-	FileSource f(PKGDATA("xtrdh171.dat"), true, new HexDecoder());
+	FileSource f("xtrdh171.dat", true, new HexDecoder());
 	XTR_DH dh(f);
 	return SimpleKeyAgreementValidate(dh);
 }
@@ -376,7 +375,7 @@ bool ValidateElGamal()
 	cout << "\nElGamal validation suite running...\n\n";
 	bool pass = true;
 	{
-		FileSource fc(PKGDATA("elgc1024.dat"), true, new HexDecoder);
+		FileSource fc("elgc1024.dat", true, new HexDecoder);
 		ElGamalDecryptor privC(fc);
 		ElGamalEncryptor pubC(privC);
 		privC.AccessKey().Precompute();
@@ -394,7 +393,7 @@ bool ValidateDLIES()
 	cout << "\nDLIES validation suite running...\n\n";
 	bool pass = true;
 	{
-		FileSource fc(PKGDATA("dlie1024.dat"), true, new HexDecoder);
+		FileSource fc("dlie1024.dat", true, new HexDecoder);
 		DLIES<>::Decryptor privC(fc);
 		DLIES<>::Encryptor pubC(privC);
 		pass = CryptoSystemValidate(privC, pubC) && pass;
@@ -417,7 +416,7 @@ bool ValidateNR()
 	cout << "\nNR validation suite running...\n\n";
 	bool pass = true;
 	{
-		FileSource f(PKGDATA("nr2048.dat"), true, new HexDecoder);
+		FileSource f("nr2048.dat", true, new HexDecoder);
 		NR<SHA>::Signer privS(f);
 		privS.AccessKey().Precompute();
 		NR<SHA>::Verifier pubS(privS);
@@ -440,7 +439,7 @@ bool ValidateDSA(bool thorough)
 
 	bool pass = true, fail;
 	{
-	FileSource fs(PKGDATA("dsa512.dat"), true, new HexDecoder());
+	FileSource fs("dsa512.dat", true, new HexDecoder());
 	GDSA<SHA>::Signer priv(fs);
 	priv.AccessKey().Precompute(16);
 	GDSA<SHA>::Verifier pub(priv);
@@ -482,10 +481,10 @@ bool ValidateDSA(bool thorough)
 	fail = pub.VerifyMessage((byte *)"xyz", 3, sig, sizeof(sig));
 	pass = pass && !fail;
 	}
-	FileSource fs1(PKGDATA("dsa1024.dat"), true, new HexDecoder());
+	FileSource fs1("dsa1024.dat", true, new HexDecoder());
 	DSA::Signer priv(fs1);
 	DSA::Verifier pub(priv);
-	FileSource fs2(PKGDATA("dsa1024b.dat"), true, new HexDecoder());
+	FileSource fs2("dsa1024b.dat", true, new HexDecoder());
 	DSA::Verifier pub1(fs2);
 	assert(pub.GetKey() == pub1.GetKey());
 	pass = SignatureValidate(priv, pub, thorough) && pass;
@@ -498,7 +497,7 @@ bool ValidateLUC()
 	bool pass=true;
 
 	{
-		FileSource f(PKGDATA("luc1024.dat"), true, new HexDecoder);
+		FileSource f("luc1024.dat", true, new HexDecoder);
 		LUCSSA_PKCS1v15_SHA_Signer priv(f);
 		LUCSSA_PKCS1v15_SHA_Verifier pub(priv);
 		pass = SignatureValidate(priv, pub) && pass;
@@ -515,14 +514,14 @@ bool ValidateLUC_DL()
 {
 	cout << "\nLUC-HMP validation suite running...\n\n";
 
-	FileSource f(PKGDATA("lucs512.dat"), true, new HexDecoder);
+	FileSource f("lucs512.dat", true, new HexDecoder);
 	LUC_HMP<SHA>::Signer privS(f);
 	LUC_HMP<SHA>::Verifier pubS(privS);
 	bool pass = SignatureValidate(privS, pubS);
 
 	cout << "\nLUC-IES validation suite running...\n\n";
 
-	FileSource fc(PKGDATA("lucc512.dat"), true, new HexDecoder);
+	FileSource fc("lucc512.dat", true, new HexDecoder);
 	LUC_IES<>::Decryptor privC(fc);
 	LUC_IES<>::Encryptor pubC(privC);
 	pass = CryptoSystemValidate(privC, pubC) && pass;
@@ -536,7 +535,7 @@ bool ValidateRabin()
 	bool pass=true;
 
 	{
-		FileSource f(PKGDATA("rabi1024.dat"), true, new HexDecoder);
+		FileSource f("rabi1024.dat", true, new HexDecoder);
 		RabinSS<PSSR, SHA>::Signer priv(f);
 		RabinSS<PSSR, SHA>::Verifier pub(priv);
 		pass = SignatureValidate(priv, pub) && pass;
@@ -553,7 +552,7 @@ bool ValidateRW()
 {
 	cout << "\nRW validation suite running...\n\n";
 
-	FileSource f(PKGDATA("rw1024.dat"), true, new HexDecoder);
+	FileSource f("rw1024.dat", true, new HexDecoder);
 	RWSS<PSSR, SHA>::Signer priv(f);
 	RWSS<PSSR, SHA>::Verifier pub(priv);
 
@@ -565,7 +564,7 @@ bool ValidateBlumGoldwasser()
 {
 	cout << "\nBlumGoldwasser validation suite running...\n\n";
 
-	FileSource f(PKGDATA("blum512.dat"), true, new HexDecoder);
+	FileSource f("blum512.dat", true, new HexDecoder);
 	BlumGoldwasserPrivateKey priv(f);
 	BlumGoldwasserPublicKey pub(priv);
 
@@ -735,7 +734,7 @@ bool ValidateESIGN()
 		"\x74\x02\x37\x0E\xED\x0A\x06\xAD\xF4\x15\x65\xB8\xE1\xD1\x45\xAE\x39\x19\xB4\xFF\x5D\xF1\x45\x7B\xE0\xFE\x72\xED\x11\x92\x8F\x61\x41\x4F\x02\x00\xF2\x76\x6F\x7C"
 		"\x79\xA2\xE5\x52\x20\x5D\x97\x5E\xFE\x39\xAE\x21\x10\xFB\x35\xF4\x80\x81\x41\x13\xDD\xE8\x5F\xCA\x1E\x4F\xF8\x9B\xB2\x68\xFB\x28";
 
-	FileSource keys(PKGDATA("esig1536.dat"), true, new HexDecoder);
+	FileSource keys("esig1536.dat", true, new HexDecoder);
 	ESIGN<SHA>::Signer signer(keys);
 	ESIGN<SHA>::Verifier verifier(signer);
 
